/*
  gci.h - general case insensitive table, part of libtrish

  Copyright (C) 1988-1994,1996-1999,2003,2004 Free Software Foundation, Inc.
  This file is part of the GNU C Library.  Its master source is NOT part of
  the C library, however.  The master source lives in /gd/gnu/lib.

  The GNU C Library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  The GNU C Library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with the GNU C Library; if not, write to the Free
  Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
  02111-1307 USA.
*/

#ifndef HAVE_GCI_H
#define HAVE_GCI_H

#include "patricia.h"


/*
  gci_new - creates a new gci table
  nwc - expeced number of characters in the table
  nalt - expected total number of alternatives
  return value - the new gci, which is really a patricia tree

  Note: the gci must be freed with patricia_free
*/

patricia_t *gci_new(unsigned int nwc, unsigned int nalt);


/*
  gci_add - adds a character and its alternatives to a gci
  gci - gci table to complete
  wc - character
  alt - L'\0' terminated string with all alternatives to wc
  return value - 1 on success, 0 if the character has already alternatives
 */

int gci_add(patricia_t *gci, wchar_t wc, wchar_t *alt);


/*
  gci_alt - get alternatives to a character
  gci - gci table
  wc - character
  return value - a L'\0' terminated string with all alternatives to wc, NULL if none is found

  Note: the return value is claimed, so it should not be freed
 */

wchar_t *gci_alt(patricia_t *gci, wchar_t wc);

#endif
