/*
  Copyright (C) 1988-1994,1996-1999,2003,2004 Free Software Foundation, Inc.
  This file is part of the GNU C Library.  Its master source is NOT part of
  the C library, however.  The master source lives in /gd/gnu/lib.

  The GNU C Library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  The GNU C Library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with the GNU C Library; if not, write to the Free
  Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
  02111-1307 USA.
*/

#include "patricia_map.h"
#include "patricia_search.h"

void *pmap_add(patricia_t *pat, wchar_t *key, void *value, int replace) {
  patricia_add_context_t ctx;
  void *old;

  patricia_add_context_init(&ctx, pat, key, PATRICIA_ADD_OPTION_START | PATRICIA_ADD_OPTION_END, NULL);
  patricia_add(&ctx, NULL);
  old = ctx.current->data;
  if (replace || (old == NULL))
    ctx.current->data = value;

  return old;
}

void *pmap_get(patricia_t *pat, wchar_t *key) {
  psearch_t *search = psearch_new(pat, PSEARCH_OPTION_FULL, NULL, 1);
  psearch_new_query(search, key);
  patricia_search(search);
  return (search->final == NULL) ? NULL : search->final->node->data;
}

pmap_multi_t *pmap_multi_new(unsigned int node_block, unsigned int string_block, bunch_t *values) {
  pmap_multi_t *pmm = calloc(1, sizeof(pmap_multi_t));
  bunch_t *data = bunch_new(sizeof(pmap_multival_t), values->block_size);

  pmm->pat = patricia_new(node_block, string_block, data);
  pmm->values = values;

  return pmm;
}

void pmap_multi_free(pmap_multi_t *pmm) {
  bunch_free(pmm->pat->data);
  patricia_free(pmm->pat);
  free(pmm);
}

void pmap_multi_add(pmap_multi_t *pmm, wchar_t *key, void *value) {
  patricia_add_context_t ctx;
  pmap_multival_t *mval = NULL;

  patricia_add_context_init(&ctx, pmm->pat, key, PATRICIA_ADD_OPTION_START | PATRICIA_ADD_OPTION_END, NULL);
  patricia_add(&ctx, NULL);
  if (value != NULL) {
    mval = bunch_claim(pmm->pat->data, 1);
    mval->value = value;
    mval->next = ctx.current->data;
  }
  ctx.current->data = mval;
}

pmap_multival_t *pmap_multi_get(pmap_multi_t *pmm, wchar_t *key) {
  psearch_t *search = psearch_new(pmm->pat, PSEARCH_OPTION_FULL, NULL, 1);
  psearch_new_query(search, key);
  patricia_search(search);
  return (search->final == NULL) ? NULL : search->final->node->data;
}

void pmap_multi_save(pmap_multi_t *pmm, int options, FILE *f) {
  patricia_save(pmm->pat, options, f);
  bunch_save(pmm->values, options, f);
}

pmap_multi_t *pmap_multi_load(FILE *f) {
  pmap_multi_t *pmm = calloc(1, sizeof(pmap_multi_t));
  int i;

  pmm->pat = patricia_load(f);
  pmm->values = bunch_load(f);
  for (i = 0; i < pmm->pat->data->block_number; i++) {
    int j;
    for (j = 0; j < pmm->pat->data->block_size - pmm->pat->data->blocks[i]->unclaimed; j++) {
      pmap_multival_t *mval = (pmap_multival_t *)pmm->pat->data->blocks[i]->data + j;
      BUNCH_CORRECT_PTR(pmm->values, mval->value);
      BUNCH_CORRECT_PTR(pmm->pat->data, mval->next);
    }
  }

  return pmm;
}
