/*
  Copyright (C) 1988-1994,1996-1999,2003,2004 Free Software Foundation, Inc.
  This file is part of the GNU C Library.  Its master source is NOT part of
  the C library, however.  The master source lives in /gd/gnu/lib.

  The GNU C Library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  The GNU C Library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with the GNU C Library; if not, write to the Free
  Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
  02111-1307 USA.
*/

#include "patricia_node.h"

#include <wchar.h>

#include "mouchard.h"

pnode_t *pnode_new(bunch_t *b, pnode_t *parent, int pnode_type, wchar_t wc) {
  pnode_t *pn;

  /* Avoid weird cases */
  if (b->element_size != sizeof(pnode_t))
    return NULL;

  if ((pn = bunch_claim(b, 1)) == NULL)
    return NULL;
  
  pn->pnode_type = pnode_type;
  pn->wc = wc;
  pn->parent = parent;

  return pn;
}


/*
  pnode_compare - compares the value of a node type and a leading character to the value a reference node
  ref - pointer to the reference node
  pnode_type - node type to compare
  wc - leading character to compare
  return value - and integer lower, equal or greater than zero respectively if pnode_type and wc has a lower, equal or greater value than ref
*/

int pnode_compare(pnode_t *ref, int pnode_type, wchar_t wc) {
  if (pnode_type == ref->pnode_type)
    if (pnode_type == PNODE_TYPE_CHAR)
      return wc - ref->wc;
    else
      return 0;
  else
    return pnode_type - ref->pnode_type;
}


pnode_t *pnode_lookup(pnode_t *ref, int pnode_type, wchar_t wc) {
  ref = ref->next;
  while (ref != NULL) {
    int d = pnode_compare(ref, pnode_type, wc);
    if (d < 0)
      ref = ref->left;
    else if (d > 0)
      ref = ref->right;
    else
      break;
  }

  return ref;
}


int pnode_insert(bunch_t *b, pnode_t *ref, int pnode_type, wchar_t wc, pnode_t **new) {
  pnode_t *parent = ref;

  if (ref->next == NULL) {
    *new = pnode_new(b, parent, pnode_type, wc);
    ref->next = *new;
    return 1;
  }

  ref = ref->next;

  while (1) {
    int d = pnode_compare(ref, pnode_type, wc);
    if (d < 0)
      if (ref->left)
        ref = ref->left;
      else {
        *new = pnode_new(b, parent, pnode_type, wc);
	ref->left = *new;
        return 1;
      }
    else if (d > 0)
      if (ref->right)
	ref = ref->right;
      else {
        *new = pnode_new(b, parent, pnode_type, wc);
	ref->right = *new;
        return 1;
      }
    else {
      *new = ref;
      return 0;
    }
  }
}

wchar_t *pnode_string(pnode_t *pn, int n, bunch_t *b) {
  wchar_t *s = (b == NULL) ? calloc(n + 1, sizeof(wchar_t)) : bunch_claim(b, n + 1);

  s[n] = L'\0';
  while ((pn != NULL) && (n > 0)) {
    int fl = (pn->follow_string == NULL) ? -1 : wcslen(pn->follow_string) - 1;
    if (fl > n)
      fl = n;
    while (fl >= 0) {
      n--;
      s[n] = pn->follow_string[fl];
      fl--;
    }
    if (pn->pnode_type == PNODE_TYPE_CHAR) {
      n--;
      s[n] = pn->wc;
    }
    pn = pn->parent;
  }
  return s;
}

