/*
  Copyright (C) 1988-1994,1996-1999,2003,2004 Free Software Foundation, Inc.
  This file is part of the GNU C Library.  Its master source is NOT part of
  the C library, however.  The master source lives in /gd/gnu/lib.

  The GNU C Library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  The GNU C Library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with the GNU C Library; if not, write to the Free
  Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
  02111-1307 USA.
*/

#include "string_collection.h"

#include <wchar.h>

patricia_t *string_collection_new(unsigned int node_block, unsigned int string_block, unsigned int collection_size) {
  return patricia_new(node_block, string_block, bunch_new(sizeof(wchar_t), collection_size));
}

bunch_t *string_collection_free(patricia_t *scoll) {
  bunch_t *data = scoll->data;
  patricia_free(scoll);
  return data;
}

void string_collection_save(patricia_t *scoll, FILE *f) {
  patricia_save(scoll, 0, f);
}

patricia_t *string_collection_load(FILE *f) {
  return patricia_load(f);
}

void string_collection_hook(const patricia_add_context_t *ctx) {
  if ((ctx->current->pnode_type == PNODE_TYPE_END) && (ctx->event == PATRICIA_ADD_EVENT_NEW_NODE)) {
    ctx->current->data = bunch_claim(ctx->pat->data, wcslen(ctx->word) + 1);
    wcscat(ctx->current->data, ctx->word);
  }
}

wchar_t *string_collection_ptr(patricia_t *scoll, const wchar_t *w) {
  patricia_add_context_t ctx;
  patricia_add_context_init(&ctx, scoll, w, PATRICIA_ADD_OPTION_START | PATRICIA_ADD_OPTION_END, NULL);
  patricia_add(&ctx, string_collection_hook);
  return ctx.current->data;
}
