#ifndef HAVE_TRKOV_DICT_H
#define HAVE_TRKOV_DICT_H

#include "patricia.h"

/*
  trk_dict_t - A Markov count structure
 */
typedef struct {
  /* kmax - Higher computed order */
  int kmax;

  /* global - Global counts */
  patricia_t *global;

  /* max_pos - Higher position found yet */
  int max_pos;

  /* positional - Position-dependent counts */
  patricia_t **positional;
} trk_dict_t;


/*
  trk_dict_new - creates a new Markov count
  nentries - estimation of the number of entries
  kmax - Higher order to compute, no limit if 0
  pos - will compute position-dependent counts if nonzero
  return value - a freshly allocated Markov count
*/

trk_dict_t *trk_dict_new(unsigned int nentries, int kmax, int pos);


/*
  trk_dict_free - destroys a Markov count
  tk - structure to destroy, must have been allocated with trk_dict_new
*/

void trk_dict_free(trk_dict_t *tk);


/*
  trk_dict_save - writes a Markov count into a file
  tk - Markov count to save
  f - file handle to save into, must have been opened in mode "w"
*/

void trk_dict_save(trk_dict_t *tk, FILE *f);


/*
  trk_dict_load - reads a Markov count from a file
  f - file to read from, must have been opened with mode "r"
  return value - a freshly allocated Markov count, filled with the file contents
  Note: the file contents must have been filled with trk_dict_save, all PATRICIAS are pruned, you cannot add any more words
 */

trk_dict_t *trk_dict_load(FILE *f);


/*
  trk_dict_add - Adds a word into the Markov count
  tk - Markov count to add to
  word - word to add
*/
void trk_dict_add(trk_dict_t *tk, wchar_t *word);


/*
  PNODE_COUNT - gets the count of a PATRICIA node
  node - the node to count
  return value - the number of occurrences of the string corresponding to the node
  Note: you cannot use this value a lvalue
 */
#define PNODE_COUNT(node) (uintptr_t)(node)->data

#endif
