/*
  wchar_utils.h - a functions for wide character strings, part of libtrish

  Copyright (C) 1988-1994,1996-1999,2003,2004 Free Software Foundation, Inc.
  This file is part of the GNU C Library.  Its master source is NOT part of
  the C library, however.  The master source lives in /gd/gnu/lib.

  The GNU C Library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  The GNU C Library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with the GNU C Library; if not, write to the Free
  Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
  02111-1307 USA.
*/

#ifndef HAVE_WCHAR_UTILS_H
#define HAVE_WCHAR_UTILS_H

#include <stdio.h>


/*
  wchar_duplicate - duplicates a wide character string
  src - wide character to copy
  return value - a freshly allocated wide character string
  Note: you free it
 */

wchar_t *wchar_duplicate(const wchar_t *src);


/*
  get_nonls - counts the number of newline characters at the end of the string
  s - string to count in
  l - length of s, will be recomputed if <= 0
  return value - number of newline characters (1 for CR, 2 for CRLF, 0 otherwise)
 */

int get_nonls(wchar_t *s, size_t l);


/*
  char2wcs - translates a multi-byte string into a wide character string
  s - string to convert
  return value - a wide character string freshly allocated, NULL if there was an error
  Note: it is your responsibility to free the returned pointer, char2wcs will use the current value of LC_CTYPE
 */

wchar_t *char2wcs(const char *s);


/*
  tabs_and_nls - replaces \t and \n by tabs and newlines
  w - string to convert
  return value - the new length of the string
*/

size_t tabs_and_nls(wchar_t *w);


/*
  ESCAPE_WCHAR - escape wide character
*/

#define ESCAPE_WCHAR L'\\'

#endif
